function [points,labels] = md_create_points(density, num, dim)
% Creates points of some densities used for testing manifold denoising as a
% denoising method and as a preprocessing step for semi-supervised
% learning.
% Parameters:
% density: the type of density used for testing. Possible choices are:
%   0: uniform distribution in two dimensions with noise
%   1: three points with different density
%   2: two lines
%   3: 2D circle
%   4: 1D sinusoid
%   5: 2D Sphere
%   6: strip
%   7: three gaussians
%   9: two-moon problem (with noise)
% num: the number of sample points
% dim: the dimensionality of the surrounding space

% initialize random number generator
randn('state',1000);
rand('state',1000);

% generate random points points from some distributions 
labels=zeros(num,1);
switch(density)
    case 0,
        % uniform distribution in in_dim dimensions with dim Gaussian noise
        in_dim=2;
        points = 6*rand(in_dim,num)-3;
        points = points*6;
        labels(1:num,1)=1;
        points=[points; zeros(dim - in_dim,num)]+0.4*randn(dim,num);

    case 1, 
        % three points with different density
        a=floor(num/3); b=floor(num/4); c=num-a-b;
        points(:,1:a)      =0.6*randn(dim,a) - [1.1*ones(2,a); zeros(dim-2,a)]; labels(1:a,1)=1;
        points(:,a+1:a+b)  =0.6*randn(dim,b) + [ones(2,b); zeros(dim-2,b)]; labels(a+1:a+b,1)=2;
        points(:,a+b+1:num)=0.6*randn(dim,c) + [2*ones(1,c); -2*ones(1,c); zeros(dim-2,c)]; labels(a+b+1:num,1)=3;
        display(['Generation of three isotropic Gaussians of equal variance in ',num2str(dim),' dimensions']);
        display(['Number of points in Cluster 1: ',num2str(a),'; Cluster 2: ',num2str(b),'; Cluster 3: ',num2str(c)]);

        
    case 2,
        % two lines
        points=[rand(1,num/2);   zeros(1,num/2); zeros(dim-2,num/2)]+0.15*randn(dim,num/2);
        labels(1:num/2,1)=2;
        points(:,num/2+1:num)=[rand(1,num/2); 0.5*ones(1,num/2); zeros(dim-2,num/2)]+0.15*randn(dim,num/2);
        labels(num/2+1:num,1)=1;

        
    case 3, 
        % 2D-circle
        t=rand(1,num); points=[sin(2*pi*t); cos(2*pi*t) ; zeros(dim-2,num)]+0.2*randn(dim,num); labels(1:num,1)=1;

        
    case 4, 
        % 1D-sinusoid
        t=-0.25+1.5*rand(1,num); points=[sin(2*pi*t); 2*pi*t;     zeros(dim-2,num)]+0.2*randn(dim,num); labels(1:num,1)=1;

        
    case 5,
        % in-dim D- sphere
        in_dim = 2;
        points=randn(in_dim+1,num); for i=1:num, points(:,i)=points(:,i)/norm(points(:,i)); end; labels(1:num,1)=1;
        points=[points ; zeros(dim - (in_dim+1),num)]+0.2*randn(dim,num);

        
    case 6, 
        % strip
        points=[10*rand(1,num); 0.1*rand(dim-1,num)-0.05*ones(dim-1,num)]; labels(1:num,1)=1;

        
    case 7, 
        % three gaussians mapped
        a=floor(num/5); b=floor(num/3); c=num-a-b;
        points(:,1:a)      =0.6*randn(dim,a) - [ones(2,a) ; zeros(dim-2,a)]; labels(1:a,1)=1;
        points(:,a+1:a+b)  =0.6*randn(dim,b) + [ones(2,b) ; zeros(dim-2,b)]; labels(a+1:a+b,1)=2;
        points(:,a+b+1:num)=0.6*randn(dim,c) + [zeros(1,c); -2*ones(1,c);zeros(dim-2,c)]; labels(a+b+1:num,1)=3;
        x_min=min(points,2); x_max=max(points,2);
        points(1,:)=(points(1,:)-x_min(1))/(x_max(1)-x_min(1)); points(2,:)=(points(2,:)-x_min(2))/(x_max(2)-x_min(2));
        points=[sin(2*pi*points(1,:)).*sin(pi*points(2,:)); cos(2*pi*points(1,:)).*sin(pi*points(2,:)); cos(pi*points(2,:))];

        
    case 9, 
        % two-moon problem
        num_pos=floor(num/2); num_neg=num-num_pos;
        radii=ones(num,1);
        phi  =rand(num_pos+num_neg,1).*pi;
        points=zeros(dim,num);
        for i=1:num_pos
            points(1,i)=radii(i)*cos(phi(i));
            points(2,i)=radii(i)*sin(phi(i));
            labels(i,1)=-1;
        end
        for i=num_pos+1:num_neg+num_pos
            points(1,i)=1+radii(i)*cos(phi(i));
            points(2,i)=-radii(i)*sin(phi(i))+0.5;
            labels(i,1)=1;
        end
        points=points + 0.2*randn(dim,num);
        
end
