function [x,KNN,KNNDist,K] = md_time_step(x,num_knn,lambda,timestep,GraphType)
% Executes one time step of manifold denoising as proposed in the NIPS'06
% paper. The parameters are:
%   x: the points
%   num_knn: the number of nearest neighbors
%   lambda: reweighting of the weights
%   timestep: 0 for explicit timestep; 1 for implicit timestep
%   GraphType: 0 for mutual kNN graph; 1 for symmetric kNN graph
% The return values are:
%   x: the new coordinates of the points
%   KNN: the nearest neighbors of each point
%   KNNDist: the distance to the k-nearest neighbor
%   K: the weight matrix used in the construction of the Laplacian

% setting some constants
% type of Laplacian (normalized, unnormalized)
laplacian = 0;
% compact support of kernel
R = 1.0;

% get dimensions of points
[dim num] = size(x);

% compute the KNN directly
tic, [KNN,KNNDist]=getKNN(x,num_knn); t=toc;
display(['Average KNN-Dist: ',num2str(mean(KNNDist(num_knn,:))),'; Standard deviation: ',num2str(std(KNNDist(num_knn,:))),'; Minimum: ',...
    num2str(min(KNNDist(num_knn,:))),'; Maximum: ',num2str(max(KNNDist(num_knn,:)))]);
disp(['Time to build kNN graph: ', num2str(t),' seconds']);

% first build the weight matrix
tic;
% arguments the knn-Distances and the list KNN with the k nearest
% neigbhors manifold denoising
KNN=double(KNN); ixx=1:num; ixx=repmat(ixx,num_knn,1);
K=sparse(ixx,KNN,KNNDist);
%K = getSparseWeightMatrixFromKNN(KNNDist,KNN,R,GraphType);
% note that K is not symmetric yet (in the case of the symmetric KNN), now we symmetrize K
if(GraphType==0)
 K=min(K,K');
end
if(GraphType==1)
 K=max(K,K');
end
t=toc;
disp(['Time to build Weight Matrix: ', num2str(t),' seconds']);

% KNNDist can be cleared, if it is not to be returned
if(nargout<3)
    clear KNNDist;
end

% build the degree function ( d_i = sum_j w_ij)
d=sum(K,2);

% checks if elements of the degree function are zero and correct this
if(sum(d==0)>0)
    disp('Warning, Elements of d are zero');
    for i=1:num
        if(d(i)==0), d(i)=1; end
    end
end

% build the final weight matrix - reweighted by some power of the degree
% function \tilde{k}_ij = k_ij / pow(d_i d_j, lambda)
if(lambda~=0)
    f=spdiags(1./(d.^lambda), 0, num, num);
    K=1/(num)*f*K*f;
    clear f;
end
clear d

% final degree function of weights \tilde{k}
e=sum(K,2);
if(sum(e==0)>0)
    disp('Warning, Elements of e are zero');
    for i=1:num
        if(e(i)==0), e(i)=1/(num); end
    end
end

% for the time step it is easier to have the transpose of x
x=x';

tic
if(timestep==0)
    % explicit timestep
    E=spdiags(e,0,num,num);
    L=E-K;
    step=-0.1*L*x;
    x = x + step;
else
    % implicit timestep
    E=spdiags(e,0,num,num);
    L=E-K;
    switch laplacian
        case 0,
            %normalized, solve L*x_new =x_old, where L=Id - E^{-1}W
            % we have multiplied here the whole equation with E to
            % transform the problem into a symmetric one
            % Problems could arise if e is zero
            % somewhere, which we explicitly avoid above
            %z=cholmod(E + 0.5*(E-K), E*x);
            z=(E+0.25*L)\(E*x);
            diff = z - x;
            for i=1:num
                x(i,:) = x(i,:) + diff(i,:);
            end
        case 1,
            %unnormalized
            % z=cholmod(speye(num) + 0.5*L,x);
            z = (speye(num) + 0.5*L)\x;
            diff = z - x;
            for i=1:num
                x(i,:) = x(i,:) + diff(i,:);
            end
    end
end
if(nargout<4)
    clear K;
end

x=x'; % transform the data back in column format (one data point=one column in x)
t=toc;
disp(['Time for time step: ', num2str(t),' seconds']);
